@@ -38,6 +38,12 @@ struct _RsttoPictureViewerPriv
     GTimer           *timer;
     gint              timeout_id;
     gboolean          timeout;
+    struct
+    {
+        gdouble x;
+        gdouble y;
+    } motion;
+    GtkMenu          *menu;
 };
 
 static void
@@ -70,6 +76,17 @@ static void
 cb_rstto_picture_viewer_value_changed(GtkAdjustment *, RsttoPictureViewer *);
 static void
 cb_rstto_picture_viewer_scroll_event (RsttoPictureViewer *, GdkEventScroll *);
+static gboolean 
+cb_rstto_picture_viewer_motion_notify_event (RsttoPictureViewer *viewer,
+                                             GdkEventMotion *event,
+                                             gpointer user_data);
+static void
+cb_rstto_picture_viewer_button_press_event (RsttoPictureViewer *viewer, GdkEventButton *event);
+static void
+cb_rstto_picture_viewer_button_release_event (RsttoPictureViewer *viewer, GdkEventButton *event);
+static void
+cb_rstto_picture_viewer_popup_menu (RsttoPictureViewer *viewer, gboolean user_data);
+
 static gboolean
 cb_rstto_picture_viewer_update_image(RsttoPictureViewer *viewer);
 
@@ -121,12 +138,20 @@ rstto_picture_viewer_init(RsttoPictureViewer *viewer)
     viewer->priv->src_pixbuf = NULL;
     viewer->priv->dst_pixbuf = NULL;
     gtk_widget_set_redraw_on_allocate(GTK_WIDGET(viewer), TRUE);
+    gtk_widget_set_events (GTK_WIDGET(viewer),
+                           GDK_BUTTON_PRESS_MASK |
+                           GDK_BUTTON_RELEASE_MASK |
+                           GDK_BUTTON1_MOTION_MASK);
 
     viewer->priv->show_border = TRUE;
     viewer->priv->timer = g_timer_new();
     viewer->priv->timeout = TRUE;
 
     g_signal_connect(G_OBJECT(viewer), "scroll_event", G_CALLBACK(cb_rstto_picture_viewer_scroll_event), NULL);
+    g_signal_connect(G_OBJECT(viewer), "button_press_event", G_CALLBACK(cb_rstto_picture_viewer_button_press_event), NULL);
+    g_signal_connect(G_OBJECT(viewer), "button_release_event", G_CALLBACK(cb_rstto_picture_viewer_button_release_event), NULL);
+    g_signal_connect(G_OBJECT(viewer), "motion_notify_event", G_CALLBACK(cb_rstto_picture_viewer_motion_notify_event), NULL);
+    g_signal_connect(G_OBJECT(viewer), "popup-menu", G_CALLBACK(cb_rstto_picture_viewer_popup_menu), NULL);
 }
 
 static void
@@ -161,7 +186,6 @@ rstto_picture_viewer_class_init(RsttoPictureViewerClass *viewer_class)
                                 G_TYPE_NONE, 2,
                                 GTK_TYPE_ADJUSTMENT,
                                 GTK_TYPE_ADJUSTMENT);
-
 }
 
 static void
@@ -588,7 +612,7 @@ rstto_picture_viewer_refresh(RsttoPictureViewer *viewer)
                 viewer->vadjustment->page_increment = 100;
                 if((viewer->vadjustment->value + viewer->vadjustment->page_size) > viewer->vadjustment->upper)
                 {
-                    viewer->vadjustment->value = viewer->vadjustment->upper - viewer->vadjustment->page_size - 1;
+                    viewer->vadjustment->value = viewer->vadjustment->upper - viewer->vadjustment->page_size;
                     vadjustment_changed = TRUE;
                 }
             }
@@ -913,26 +937,49 @@ cb_rstto_picture_viewer_closed(GdkPixbufLoader *loader, RsttoPictureViewer *view
 {
     GtkWidget *widget = GTK_WIDGET(viewer);
 
+    RsttoNavigatorEntry *entry = rstto_navigator_get_file(viewer->priv->navigator);
+
     if (viewer->priv->src_pixbuf)
     {
         gdk_pixbuf_unref(viewer->priv->src_pixbuf);
         viewer->priv->src_pixbuf = NULL;
     }
-    if (viewer->priv->iter)
+
+    if (entry)
     {
-        viewer->priv->src_pixbuf = gdk_pixbuf_animation_iter_get_pixbuf(viewer->priv->iter);
-        viewer->priv->src_pixbuf = gdk_pixbuf_copy(viewer->priv->src_pixbuf);
-    }
-    else
-    {
-        if (viewer->priv->loader)
+        GdkPixbuf *pixbuf = NULL;
+
+        if (viewer->priv->iter)
         {
-            viewer->priv->src_pixbuf = gdk_pixbuf_loader_get_pixbuf(viewer->priv->loader);
-            if (viewer->priv->src_pixbuf)
+            pixbuf = gdk_pixbuf_animation_iter_get_pixbuf(viewer->priv->iter);
+        }
+        else
+        {
+            if (viewer->priv->loader)
             {
-                gdk_pixbuf_ref(viewer->priv->src_pixbuf);
+                pixbuf = gdk_pixbuf_loader_get_pixbuf(viewer->priv->loader);
             }
         }
+
+       
+        if (pixbuf != NULL)
+        {
+            viewer->priv->src_pixbuf = gdk_pixbuf_rotate_simple(pixbuf, rstto_navigator_entry_get_rotation(entry));
+            if (rstto_navigator_entry_get_flip(entry, FALSE))
+            {
+                pixbuf = viewer->priv->src_pixbuf;
+                viewer->priv->src_pixbuf = gdk_pixbuf_flip(pixbuf, FALSE);
+                gdk_pixbuf_unref(pixbuf);
+            }
+
+            if (rstto_navigator_entry_get_flip(entry, TRUE))
+            {
+                pixbuf = viewer->priv->src_pixbuf;
+                viewer->priv->src_pixbuf = gdk_pixbuf_flip(pixbuf, TRUE);
+                gdk_pixbuf_unref(pixbuf);
+            }
+        }
+        
     }
 
     rstto_picture_viewer_refresh(viewer);
@@ -942,3 +989,111 @@ cb_rstto_picture_viewer_closed(GdkPixbufLoader *loader, RsttoPictureViewer *view
         gdk_window_set_cursor(widget->window, NULL);
     }
 }
+
+static gboolean 
+cb_rstto_picture_viewer_motion_notify_event (RsttoPictureViewer *viewer,
+                                             GdkEventMotion *event,
+                                             gpointer user_data)
+{
+    if (event->state & GDK_BUTTON1_MASK)
+    {
+    }
+    return TRUE;
+}
+
+static void
+cb_rstto_picture_viewer_button_press_event (RsttoPictureViewer *viewer, GdkEventButton *event)
+{
+    if(event->button == 1)
+    {
+        GtkWidget *widget = GTK_WIDGET(viewer);
+        GdkCursor *cursor = gdk_cursor_new(GDK_FLEUR);
+        gdk_window_set_cursor(widget->window, cursor);
+        gdk_cursor_unref(cursor);
+        
+        viewer->priv->motion.x = event->x;
+        viewer->priv->motion.y = event->y;
+    }
+    if(event->button == 3)
+    {
+        if (viewer->priv->menu)
+        {
+            gtk_widget_show_all(GTK_WIDGET(viewer->priv->menu));
+            gtk_menu_popup(viewer->priv->menu,
+                           NULL,
+                           NULL,
+                           NULL,
+                           NULL,
+                           3,
+                           event->time);
+        }
+    }
+}
+
+static void
+cb_rstto_picture_viewer_button_release_event (RsttoPictureViewer *viewer, GdkEventButton *event)
+{
+    if(event->button == 1)
+    {
+        /* Move the image around */
+        GtkWidget *widget = GTK_WIDGET(viewer);
+        GdkCursor *cursor = gdk_cursor_new(GDK_WATCH);
+        gdk_window_set_cursor(widget->window, cursor);
+        gdk_cursor_unref(cursor);
+
+        if (viewer->priv->motion.x != event->x)
+        {
+            viewer->hadjustment->value += (viewer->priv->motion.x - event->x);
+            if((viewer->hadjustment->value + viewer->hadjustment->page_size) > viewer->hadjustment->upper)
+            {
+                viewer->hadjustment->value = viewer->hadjustment->upper - viewer->hadjustment->page_size;
+            }
+            gtk_adjustment_value_changed(viewer->hadjustment);
+        }
+        if (viewer->priv->motion.y != event->y)
+        {
+            viewer->vadjustment->value += (viewer->priv->motion.y - event->y);
+            if((viewer->vadjustment->value + viewer->vadjustment->page_size) > viewer->vadjustment->upper)
+            {
+                viewer->vadjustment->value = viewer->vadjustment->upper - viewer->vadjustment->page_size;
+            }
+            gtk_adjustment_value_changed(viewer->vadjustment);
+        }
+
+        gdk_window_set_cursor(widget->window, NULL);
+    }
+
+}
+
+static void
+cb_rstto_picture_viewer_popup_menu (RsttoPictureViewer *viewer, gboolean user_data)
+{
+    if (viewer->priv->menu)
+    {
+        gtk_widget_show_all(GTK_WIDGET(viewer->priv->menu));
+        gtk_menu_popup(viewer->priv->menu,
+                       NULL,
+                       NULL,
+                       NULL,
+                       NULL,
+                       0,
+                       gtk_get_current_event_time());
+    }
+}
+
+void
+rstto_picture_viewer_set_menu (RsttoPictureViewer *viewer, GtkMenu *menu)
+{
+    if (viewer->priv->menu)
+    {
+        gtk_menu_detach(viewer->priv->menu);
+        gtk_widget_destroy(GTK_WIDGET(viewer->priv->menu));
+    }
+
+    viewer->priv->menu = menu;
+
+    if (viewer->priv->menu)
+    {
+        gtk_menu_attach_to_widget(viewer->priv->menu, GTK_WIDGET(viewer), NULL);
+    }
+}
